# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
""" Copyright (c) 2002-2003 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr

module manipulation utilities
"""

__revision__ = "$Id: modutils.py,v 1.4 2003/06/12 08:04:03 syt Exp $"

import os
import sys
import inspect
from os.path import walk, splitext, join, abspath, isabs, isdir, dirname, exists

# get standard library directory for the running python version
STD_LIB_DIR = join(sys.prefix, 'lib', 'python%s' % sys.version[:3])


def load_module_from_name(dotted_name, globals=None):
    """ load a python module from it's name """
    mod = __import__(dotted_name, globals)
    components = dotted_name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod


def modpath_from_file(filename):
    """ given an absolute file path return the python module's path as a list """
    base = splitext(abspath(filename))[0]
    for path in sys.path:
        if path and base[:len(path)] == path:
            if filename.find('site-packages') != -1 and path.find('site-packages') == -1:
                continue
            mod_path = base[len(path):].split(os.sep)
            mod_path = [p for p in mod_path if p]
            break
    else:
        raise Exception('Unable to find module for %s in %s' % (
            base, ', \n'.join(sys.path)))
    return mod_path


def get_module_part(dotted_name, globals=None):
    """ given a dotted name like 'logilab.common.modutils.get_module',
    return the module part of the name (in the  previous example,
    'logilab.common.modutils' would be returned)
    """
    try:
        mod = __import__(dotted_name, globals)
        # if this import succeed then dotted_name is a module
        return dotted_name
    except ImportError:
        dotted_name = '.'.join(dotted_name.split('.')[:-1])
        mod = __import__(dotted_name, globals)
        return dotted_name

    
def is_standard_module(modname, std_path=(STD_LIB_DIR,), globals=None):
    """
    return true if the module may be considered as a module from the standard
    library
    """
    try:
        module = load_module_from_name(modname, globals)
    except ImportError:
        return 0
    try:
        file = module.__file__
    except AttributeError:
        # builtin modules don't have the __file__ attribute
        return 1
    for path in std_path:
        pfx_len = len(path)
        if file[:pfx_len] == path:
            if file[pfx_len+1:pfx_len+14] != 'site-packages':
                return 1
            return 0
    return 0


def is_relative(modname, from_file):
    """
    return true if the module may be considered as a module from the standard
    library
    """
    package = modname.split('.')[0]
    if isdir(from_file):
        mod_or_pack = join(from_file, package)
    else:
        mod_or_pack = join(dirname(from_file), package)
    # is it a relative module ?
    for ext in ('.py', '.pyc', '.pyo', '.pyd', '.so'):
        if exists(mod_or_pack + ext):
            return 1
    # is it a relative package ?
    mod_or_pack = join(mod_or_pack, '__init__')
    for ext in ('.py', '.pyc', '.pyo', '.pyd'):
        if exists(mod_or_pack + ext):
            return 1
    # no, so it must be absolute
    return 0


def get_modules(package, src_directory, blacklist = ('CVS','debian')):
    """ given a directory return a list of all available python modules, even
    in subdirectories
    """
    def func(modules, directory, fnames):
        # remove files/directories in the black list
        for norecurs in blacklist:
            try:
                fnames.remove(norecurs)
            except:
                pass
        # check for __init__.py
        if not '__init__.py' in fnames:
            while fnames:
                fnames.pop()
        elif directory != src_directory:
            #src = join(directory, file)
            modules.append(package + directory[len(src_directory):].replace(os.sep, '.'))
            
        for file in fnames:
            src = join(directory, file)
            if isdir(src):
                continue
            if file[-3:] in ('.py', '.so') and file != '__init__.py':
                module = package + src[len(src_directory):-3]
                modules.append(module.replace(os.sep, '.'))
                
    modules = []
    walk(src_directory, func, modules)
    return modules
